@@ -1,5 +1,5 @@
 from __future__ import absolute_import
-# Copyright (c) 2010-2018 openpyxl
+# Copyright (c) 2010-2019 openpyxl
 
 """
 Collection of utilities used within the package and also available for client code
@@ -10,7 +10,7 @@ from openpyxl.compat import basestring
 from .exceptions import CellCoordinatesException
 
 # constants
-COORD_RE = re.compile(r'^[$]?([A-Z]+)[$]?(\d+)$')
+COORD_RE = re.compile(r'^[$]?([A-Za-z]{1,3})[$]?(\d+)$')
 COL_RANGE = """[A-Z]{1,3}:[A-Z]{1,3}:"""
 ROW_RANGE = r"""\d+:\d+:"""
 RANGE_EXPR = r"""
@@ -28,7 +28,7 @@ SHEETRANGE_RE = re.compile("""{0}(?P<cells>{1})(?=,?)""".format(
 
 def get_column_interval(start, end):
     """
-    Given the start and end colums, return all the columns in the series.
+    Given the start and end columns, return all the columns in the series.
 
     The start and end columns can be either column letters or 1-based
     indexes.
@@ -42,7 +42,7 @@ def get_column_interval(start, end):
 
 def coordinate_from_string(coord_string):
     """Convert a coordinate string like 'B12' to a tuple ('B', 12)"""
-    match = COORD_RE.match(coord_string.upper())
+    match = COORD_RE.match(coord_string)
     if not match:
         msg = 'Invalid cell coordinates (%s)' % coord_string
         raise CellCoordinatesException(msg)
@@ -51,14 +51,15 @@ def coordinate_from_string(coord_string):
     if not row:
         msg = "There is no row 0 (%s)" % coord_string
         raise CellCoordinatesException(msg)
-    return (column, row)
+    return column, row
 
 
 def absolute_coordinate(coord_string):
     """Convert a coordinate to an absolute coordinate string (B12 -> $B$12)"""
-    m = ABSOLUTE_RE.match(coord_string.upper())
+    m = ABSOLUTE_RE.match(coord_string)
     if not m:
-        raise ValueError("Value is not a valid coordinate range")
+        raise ValueError("{0} is not a valid coordinate range".format(
+            coord_string))
 
     d = m.groupdict('')
     for k, v in d.items():
@@ -66,7 +67,7 @@ def absolute_coordinate(coord_string):
             d[k] = "${0}".format(v)
 
     if d['max_col'] or d['max_row']:
-        fmt =  "{min_col}{min_row}:{max_col}{max_row}"
+        fmt = "{min_col}{min_row}:{max_col}{max_row}"
     else:
         fmt = "{min_col}{min_row}"
     return fmt.format(**d)
@@ -130,11 +131,24 @@ def range_boundaries(range_string):
     (min_col, min_row, max_col, max_row)
     Cell coordinates will be converted into a range with the cell at both end
     """
+    msg = "{0} is not a valid coordinate or range".format(range_string)
     m = ABSOLUTE_RE.match(range_string)
     if not m:
-        raise ValueError("{0} is not a valid coordinate or range")
+        raise ValueError(msg)
+
     min_col, min_row, sep, max_col, max_row = m.groups()
 
+    if sep:
+        cols = min_col, max_col
+        rows = min_row, max_row
+
+        if not (
+            all(cols + rows) or
+            all(cols) and not any(rows) or
+            all(rows) and not any(cols)
+        ):
+            raise ValueError(msg)
+
     if min_col is not None:
         min_col = column_index_from_string(min_col)
 
@@ -160,8 +174,8 @@ def rows_from_range(range_string):
     Yields one row at a time.
     """
     min_col, min_row, max_col, max_row = range_boundaries(range_string)
-    rows = range(min_row, max_row+1)
-    cols = [get_column_letter(col) for col in range(min_col, max_col+1)]
+    rows = range(min_row, max_row + 1)
+    cols = [get_column_letter(col) for col in range(min_col, max_col + 1)]
     for row in rows:
         yield tuple('{0}{1}'.format(col, row) for col in cols)
 
@@ -182,8 +196,9 @@ def coordinate_to_tuple(coordinate):
     """
     Convert an Excel style coordinate to (row, colum) tuple
     """
-    col, row = coordinate_from_string(coordinate)
-    return row, _COL_STRING_CACHE[col]
+    match = COORD_RE.split(coordinate)
+    col, row = match[1:3]
+    return int(row), _COL_STRING_CACHE[col]
 
 
 def range_to_tuple(range_string):
@@ -204,11 +219,8 @@ def quote_sheetname(sheetname):
     """
     Add quotes around sheetnames if they contain spaces.
     """
-    pattern = re.compile(r"^(\d|\.)|([-%'(){};\s\"])")
     if "'" in sheetname:
         sheetname = sheetname.replace("'", "''")
 
-    if pattern.search(sheetname):
-        sheetname = u"'{0}'".format(sheetname)
-
+    sheetname = u"'{0}'".format(sheetname)
     return sheetname
